import yaml
from stix2 import Bundle, Vulnerability, ExternalReference, TLP_AMBER

from titan_client.titan_stix.mappers.common import BaseMapper, StixMapper, generate_id, author_identity


@StixMapper.register("cves", lambda x: "cveReportsTotalCount" in x)
@StixMapper.register("cve", lambda x: "cve_report" in x.get("data", {}))
class CveMapper(BaseMapper):

    def map(self, source: dict, girs_names: dict = None) -> Bundle:
        container = {}
        items = source.get("cveReports") or [] if "cveReportsTotalCount" in source else [source]
        for item in items:
            uid = item["uid"]
            name = item["data"]["cve_report"]["name"]
            summary = item["data"]["cve_report"]["summary"]
            underground_activity_summary = item["data"]["cve_report"]["underground_activity_summary"]
            extras = yaml.dump({k: v for k, v in item["data"]["cve_report"].items() if k in (
                "vendor_name", "product_name", "patch_status", "interest_level", "activity_location", "exploit_status")})
            girs_paths = item["classification"]["intel_requirements"]
            girs_names = girs_names or {}
            girs = [{"path": i, "name": girs_names.get(i)} for i in girs_paths]
            description = f"{summary}\n\n{underground_activity_summary}\n\n### Properties\n\n```yaml\n{extras}```" \
                          f"\n\n### Intel requirements\n\n```yaml\n{yaml.dump(girs)}```"
            cvss3_score = (item["data"]["cve_report"].get("cvss_score") or {}).get("v3")
            external_references = []
            for link_type, key in (("Titan URL", "titan_links"), ("PoC", "poc_links"), ("Patch", "patch_links")):
                for link in item["data"]["cve_report"].get(key) or []:
                    external_reference = ExternalReference(
                        source_name=f"[{link_type}] {link['title']}",
                        url=link['url']
                    )
                    external_references.append(external_reference)

            custom_properties = {"x_intel471_com_uid": uid}
            if cvss3_score:
                custom_properties["x_opencti_base_score"] = cvss3_score
            vulnerability = Vulnerability(id=generate_id(Vulnerability, name=name.strip().lower()),
                                          name=name,
                                          description=description,
                                          created_by_ref=author_identity,
                                          external_references=external_references,
                                          object_marking_refs=[TLP_AMBER],
                                          custom_properties=custom_properties)
            container[vulnerability.id] = vulnerability
            container[author_identity.id] = author_identity
            container[TLP_AMBER.id] = TLP_AMBER
        if container:
            bundle = Bundle(*container.values(), allow_custom=True)
            return bundle



import datetime
import logging

from pytz import UTC
from stix2 import Bundle, Indicator, Report, TLP_AMBER

from titan_client.titan_stix.patterning import STIXPatterningMapper
from titan_client.titan_stix.mappers.reports import ReportMapper
from titan_client.titan_stix.mappers.common import StixMapper, BaseMapper, generate_id, author_identity

log = logging.getLogger(__name__)


@StixMapper.register("iocs", lambda x: "iocTotalCount" in x)
class IOCMapper(BaseMapper):

    def map(self, source: dict, girs_names: dict = None) -> Bundle:
        container = {}
        items = source.get("iocs") or [] if "iocTotalCount" in source else [source]
        for item in items:
            ioc_type = item["type"]
            ioc_value = item["value"]
            ioc_id = item["uid"]
            report_sources = item["links"].get("reports") or []
            valid_from = datetime.datetime.fromtimestamp(item["activeFrom"] / 1000, UTC)
            valid_until = datetime.datetime.fromtimestamp(item["activeTill"] / 1000, UTC)
            if valid_from == valid_until:
                valid_until = None

            pattern_mapper = getattr(STIXPatterningMapper, f"map_{ioc_type}", None)
            if pattern_mapper:
                stix_pattern = pattern_mapper(ioc_value)
                indicator = Indicator(id=generate_id(Indicator, pattern=stix_pattern),
                                      indicator_types=["malicious-activity"],
                                      pattern_type="stix",
                                      pattern=stix_pattern,
                                      valid_from=valid_from,
                                      valid_until=valid_until,
                                      created_by_ref=author_identity,
                                      object_marking_refs=[TLP_AMBER],
                                      custom_properties={"x_intel471_com_uid": ioc_id})

                container[indicator.id] = indicator
                container[TLP_AMBER.id] = TLP_AMBER
                for uid, stix_object in self.map_reports(report_sources, indicator).items():
                    if isinstance(stix_object, Report) and uid in container:
                        stix_object.object_refs.extend(container[uid].object_refs)
                    container[uid] = stix_object
        if container:
            bundle = Bundle(*container.values(), allow_custom=True)
            return bundle

    def map_reports(self, report_sources: list, indicator: Indicator) -> dict:
        container = {}
        report_mapper = ReportMapper()
        for report_source in report_sources:
            container.update(report_mapper.map_reports(report_source, object_refs={indicator.id: indicator}))
        return container



import datetime

import yaml
from pytz import UTC
from stix2 import Indicator, Bundle, Relationship, Malware, KillChainPhase, TLP_AMBER

from titan_client.titan_stix.mappers.common import StixMapper, BaseMapper, generate_id, author_identity
from titan_client.titan_stix.patterning import STIXPatterningMapper


@StixMapper.register("indicators", lambda x: "indicatorTotalCount" in x)
class IndicatorsMapper(BaseMapper):

    def map(self, source: dict, girs_names: dict = None) -> Bundle:
        container = {}
        items = source.get("indicators") or [] if "indicatorTotalCount" in source else [source]
        for item in items:
            malware_family_name = item["data"]["threat"]["data"]["family"]
            malware_family_uid = item["data"]["threat"]["data"]["malware_family_profile_uid"]
            valid_from = datetime.datetime.fromtimestamp(item["activity"]["first"] / 1000, UTC)
            valid_until = datetime.datetime.fromtimestamp(item["data"]["expiration"] / 1000, UTC)
            tactics = self.map_tactic(item["data"]["mitre_tactics"])
            indicator_id = item["uid"]
            indicator_type = item["data"]["indicator_type"]
            indicator_data = item["data"]["indicator_data"]
            confidence = self.map_confidence(item["data"]["confidence"])
            girs_paths = item["data"]["intel_requirements"]
            girs_names = girs_names or {}
            girs = [{"path": i, "name": girs_names.get(i)} for i in girs_paths]
            description_main = item["data"]["context"]["description"]
            description = f"{description_main}\n\n### Intel requirements\n\n```yaml\n{yaml.dump(girs)}```"

            pattern_mapper = getattr(STIXPatterningMapper, f"map_{indicator_type}", None)
            if pattern_mapper:
                stix_pattern = pattern_mapper(indicator_data)

                malware = Malware(id=generate_id(Malware, name=malware_family_name.strip().lower()),
                                  name=malware_family_name,
                                  is_family=True,
                                  created_by_ref=author_identity,
                                  object_marking_refs=[TLP_AMBER],
                                  custom_properties={"x_intel471_com_uid": malware_family_uid})
                indicator = Indicator(id=generate_id(Indicator, pattern=stix_pattern),
                                      description=description,
                                      indicator_types=["malicious-activity"],
                                      pattern_type="stix",
                                      pattern=stix_pattern,
                                      valid_from=valid_from,
                                      valid_until=valid_until,
                                      kill_chain_phases=[KillChainPhase(kill_chain_name="mitre-attack", phase_name=tactics)],
                                      created_by_ref=author_identity,
                                      confidence=confidence,
                                      object_marking_refs=[TLP_AMBER],
                                      custom_properties={"x_intel471_com_uid": indicator_id})
                r1 = Relationship(indicator, "indicates", malware, created_by_ref=author_identity)
                for stix_object in [malware, indicator, author_identity, r1, TLP_AMBER]:
                    container[stix_object.id] = stix_object
        if container:
            bundle = Bundle(*container.values(), allow_custom=True)
            return bundle



import datetime

import yaml
from pytz import UTC
from stix2 import Indicator, Bundle, Relationship, Malware, TLP_AMBER
from titan_client.titan_stix.mappers.common import StixMapper, BaseMapper, generate_id, author_identity


@StixMapper.register("yara", lambda x: "yaraTotalCount" in x)
class YaraMapper(BaseMapper):

    def map(self, source: dict, girs_names: dict = None) -> Bundle:
        container = {}
        items = source.get("yaras") or [] if "yaraTotalCount" in source else [source]
        for item in items:
            yara_uid = item["uid"]
            yara_signature = item["data"]["yara_data"]["signature"]
            malware_family_name = item["data"]["threat"]["data"]["family"]
            malware_family_uid = item["data"]["threat"]["data"]["malware_family_profile_uid"]
            valid_from = datetime.datetime.fromtimestamp(item["activity"]["first"] / 1000, UTC)
            confidence = self.map_confidence(item["data"]["confidence"])
            girs_paths = item["data"]["intel_requirements"]
            girs_names = girs_names or {}
            girs = [{"path": i, "name": girs_names.get(i)} for i in girs_paths]
            description = f"### Intel requirements\n\n```yaml\n{yaml.dump(girs)}```"

            malware = Malware(id=generate_id(Malware, name=malware_family_name.strip().lower()),
                              name=malware_family_name,
                              is_family=True,
                              created_by_ref=author_identity,
                              object_marking_refs=[TLP_AMBER],
                              custom_properties={"x_intel471_com_uid": malware_family_uid})

            indicator = Indicator(id=generate_id(Indicator, pattern=yara_signature),
                                  description=description,
                                  indicator_types=["malicious-activity"],
                                  pattern_type="yara",
                                  pattern=yara_signature,
                                  valid_from=valid_from,
                                  created_by_ref=author_identity,
                                  confidence=confidence,
                                  object_marking_refs=[TLP_AMBER],
                                  custom_properties={"x_intel471_com_uid": yara_uid})
            relationship = Relationship(indicator, "indicates", malware, created_by_ref=author_identity)
            container[malware.id] = malware
            container[indicator.id] = indicator
            container[author_identity.id] = author_identity
            container[relationship.id] = relationship
            container[TLP_AMBER.id] = TLP_AMBER
        if container:
            bundle = Bundle(*container.values(), allow_custom=True)
            return bundle



import datetime
import logging
from typing import List

from pytz import UTC
from stix2 import Bundle, Report, ExternalReference, TLP_AMBER

from titan_client.titan_stix.mappers.common import StixMapper, BaseMapper, generate_id, author_identity

log = logging.getLogger(__name__)


class AbstractReportMapper(BaseMapper):
    report_type_ov = {
        "account checking": "",
        "actor profile": "threat-actor",
        "airlines": "",
        "ajax security team": "",
        "anonymous": "",
        "atms": "",
        "banking & finance": "",
        "blackhat seo": "",
        "bulletproof hosting": "tool",
        "cashout & moneymules": "",
        "click fraud": "",
        "credit card fraud": "",
        "crypters & packers": "",
        "cryptocurrency": "",
        "database dumps": "",
        "denial of service": "tool",
        "document fraud": "",
        "drops - accounts": "",
        "drops - mail": "",
        "e-commerce": "",
        "exploit kit": "tool",
        "exploit kit - usage": "tool",
        "extortion": "",
        "gaming": "",
        "government & defense industrial base": "",
        "hack the planet": "",
        "healthcare": "",
        "industrial espionage": "",
        "infrastructure": "",
        "injects": "",
        "insider": "",
        "iot (internet of things)": "",
        "law firms & legal": "",
        "lizard squad": "",
        "malware": "malware",
        "malware - technical": "malware",
        "malware - usage": "malware",
        "mobile": "",
        "new tag1": "",
        "new tag2": "",
        "phishing": "",
        "pii & fullz": "",
        "pos": "",
        "ransomware": "",
        "resources & mining": "",
        "retail": "",
        "shipping service": "",
        "skimmers": "tool",
        "social networking": "",
        "spam": "",
        "sport industry": "",
        "syrian electronic army": "",
        "targeted attack": "",
        "tickets, hotel and travel": "",
        "tools": "tool",
        "ttps": "",
        "vulnerabilities & exploits": "vulnerability",
    }

    def map(self, source: dict, girs_names: dict = None) -> Bundle:
        container = self.map_reports(source)
        if container:
            bundle = Bundle(*container.values(), allow_custom=True)
            return bundle

    def map_reports(self, source: dict, object_refs: dict = None) -> dict:
        raise NotImplementedError

    def get_name(self, subject, *categories):
        categories = [i for i in categories if i]
        if categories:
            name = f"[{'/'.join(categories)}] {subject}"
        else:
            name = subject
        return self.shorten(name, 128)

    def map_report_types(self, tags: List[str]) -> List[str]:
        types = set()
        for tag in tags:
            ov_type = self.report_type_ov.get(tag.lower())
            if ov_type:
                types.add(ov_type)
        if not types:
            types.add("miscellaneous")
        return sorted(list(types))

    def format_published(self, value: datetime):
        """
        Formatting datetime object for use as ID contributing property in a same way as it's done by OpenCTI
        to have the same ID here and in OpenCTI.
        """
        return value.strftime("%Y-%m-%dT%H:%M:%S.%f")[:-3] + "Z"


@StixMapper.register("reports", lambda x: "reportTotalCount" in x)
@StixMapper.register("report", lambda x: "subject" in x and "portalReportUrl" in x)
class ReportMapper(AbstractReportMapper):
    def map_reports(self, source: dict, object_refs: dict = None) -> dict:
        container = {}
        items = source.get("reports") or [] if "reportTotalCount" in source else [source]
        for item in items:
            report_uid = item["uid"]
            report_family = item.get("documentFamily")
            report_type = item.get("documentType")
            report_subject = item["subject"]
            report_url = item["portalReportUrl"]
            report_types = self.map_report_types(item.get("tags") or [])
            created = datetime.datetime.fromtimestamp(item.get("released", item.get("created")) / 1000, UTC)

            collected_object_refs = {}
            if object_refs:
                collected_object_refs.update(object_refs)
            for entity_source in item.get("entities") or []:
                entity = self.map_entity(entity_source["type"], entity_source["value"])
                if entity:
                    collected_object_refs[entity.id] = entity
            for location_source in item.get("locations") or []:
                location = self.map_location(location_source.get("region"), location_source.get("country"))
                if location:
                    collected_object_refs[location.id] = location
            if collected_object_refs:
                name = self.get_name(report_subject, report_family, report_type)
                report = Report(id=generate_id(Report, name=name.strip().lower(), published=self.format_published(created)),
                                name=name,
                                report_types=report_types,
                                published=self.format_published(created),
                                object_refs=collected_object_refs.values(),
                                external_references=[ExternalReference(source_name="Titan URL", url=report_url)],
                                created_by_ref=author_identity,
                                object_marking_refs=[TLP_AMBER],
                                custom_properties={"x_intel471_com_uid": report_uid})
                container[report.id] = report
                container[author_identity.id] = author_identity
                container[TLP_AMBER.id] = TLP_AMBER
                container.update(collected_object_refs)
            else:
                log.warning(f"Can't map any entities from report {report_uid}")
        return container



import abc
import datetime
import logging
import uuid
from abc import ABC
from collections import Callable
from typing import Union

from stix2 import Relationship, EmailAddress, File, ThreatActor, IPv4Address, URL, Location, DomainName, Bundle, \
    Identity
from stix2.base import _DomainObject, _Observable
from stix2.canonicalization.Canonicalize import canonicalize

from titan_client.titan_stix.exceptions import EmptyBundle, StixMapperNotFound

NAMESPACE_OASIS = uuid.UUID("00abedb4-aa42-466c-9c01-fed23315a9b7")

log = logging.getLogger(__name__)


def generate_id(stix_class: Union[_DomainObject, Relationship, _Observable], **id_contributing_properties: str):
    if id_contributing_properties:
        name = canonicalize(id_contributing_properties, utf8=False)
        return f"{stix_class._type}--{uuid.uuid5(NAMESPACE_OASIS, name)}"
    return f"{stix_class._type}--{uuid.uuid4()}"


author_name = "Intel 471 Inc."
author_identity = Identity(
    id=generate_id(Identity, name=author_name.lower(), identity_class="organization"),
    name=author_name,
    identity_class="organization",
    created=datetime.datetime(2022, 1, 1),
    modified=datetime.datetime(2022, 1, 1)
)


class StixMapper:

    mappers = {}

    @classmethod
    def register(cls, name: str, condition) -> Callable:
        """
        Decorator used for registering mapper classes. Decorate any class derived from BaseMapper like this:

        @StixMapper.register("actors", lambda x: "actorTotalCount" in x)
        class ActorsMapper(BaseMapper):
            def map(self, source: dict) -> Bundle:
                ... my implementation ...

        @param name: unique name under which the mapper will be registered
        @param condition: callable that will be called against the source dict
                          to determine if given mapper should be used or not

        """
        def inner_wrapper(wrapped_class: Callable) -> Callable:
            if name in cls.mappers:
                log.info(f"Mapper for {name} already exists. Will replace it")
            cls.mappers[name] = (condition, wrapped_class)
            return wrapped_class
        return inner_wrapper

    @classmethod
    def map(cls, source: dict, stix_version: str = "2.1", girs_names: dict = None) -> Bundle:
        log.info(f"Initializing converter. STIX version {stix_version}.")
        for name, (condition, mapper_class) in cls.mappers.items():
            if condition(source):
                log.info(f"Mapping Titan payload for {name}.")
                mapper = mapper_class()
                bundle = mapper.map(source, girs_names)
                if bundle:
                    return bundle
                else:
                    raise EmptyBundle("STIX Mapper produced an empty bundle.")
        raise StixMapperNotFound(f"STIX Mapper for this payload is not available (keys: {', '.join(source.keys())}).")


class BaseMapper(ABC):

    def __init__(self):
        self.now = datetime.datetime.utcnow()

    @abc.abstractmethod
    def map(self, source: dict) -> Bundle:
        raise NotImplementedError

    def map_entity(self, type_: str, value: str):
        entity2stix = {
            "EmailAddress": [EmailAddress, {"value": value}],
            "SHA256": [File, {"hashes": {"SHA256": value}}],
            "Handle": [ThreatActor, {"name": value}],
            "IPAddress": [IPv4Address, {"value": value}],
            "MaliciousURL": [URL, {"value": value}],
            "MaliciousDomain": [DomainName, {"value": value}],
        }
        try:
            klass, kwargs = entity2stix[type_]
        except KeyError:
            log.warning(f"Cannot map entity. Unknown type `{type_}`")
            return None
        else:
            return klass(**kwargs)

    def map_location(self, region: str = None, country: str = None):
        region_kwargs = {}
        if region:
            # TODO: map to region-ov.
            region_kwargs["region"] = region
        if country:
            region_kwargs["country"] = country
        if region_kwargs:
            return Location(id=generate_id(Location, **region_kwargs), **region_kwargs)
        return None

    def map_confidence(self, confidence: str):
        return {"low": 15, "medium": 50, "high": 85}.get(confidence, 0)

    def map_tactic(self, tactic: str):
        if tactic and len(tactic) > 0:
            return tactic.replace("_", "-").replace(" ", "-").lower()

    @staticmethod
    def shorten(text: str, limit: int) -> str:
        if len(text) > limit:
            text = text[:limit]
            while text[-1] != " ":
                text = text[:-1]
        return text.strip()




class STIXPatterningMapper:

    @classmethod
    def wrap(cls, *components):
        return "[" + " OR ".join([f"{i} = {j}" for i, j in components]) + "]"

    # Mappers for /iocs endpoint

    @classmethod
    def map_MaliciousURL(cls, value: str) -> str:
        return cls.wrap(("url:value", f"'{value}'"))

    @classmethod
    def map_MaliciousDomain(cls, value: str) -> str:
        return cls.wrap(("domain-name:value", f"'{value}'"))

    # Mappers for /indicators endpoint

    @classmethod
    def map_url(cls, item: dict) -> str:
        value = item["url"]
        return cls.wrap(("url:value", f"'{value}'"))

    @classmethod
    def map_file(cls, item: dict) -> str:
        md5 = item["file"]["md5"]
        sha1 = item["file"]["sha1"]
        sha256 = item["file"]["sha256"]
        return cls.wrap(("file:hashes.md5", f"'{md5}'"),
                        ("file:hashes.sha1", f"'{sha1}'"),
                        ("file:hashes.sha256", f"'{sha256}'"))

    @classmethod
    def map_ipv4(cls, item: dict) -> str:
        value = item["address"]
        return cls.wrap(("ipv4-addr:value", f"'{value}'"))
